import requests
from collections import Counter
import json
import cv2
from typing import Dict, Tuple, List

from blueness import module
from blue_objects import file, path
from blue_objects.graphics import add_signature
from blue_objects.objects import signature as object_signature

from vancouver_watching import NAME, VERSION
from vancouver_watching.host import signature
from vancouver_watching import env
from vancouver_watching.logger import logger

NAME = module.name(__file__, NAME)


class Ultralytics_API:
    def __init__(
        self,
        model_id: str,
        do_dryrun: bool = False,
        verbose: bool = False,
        render_inference: bool = True,
    ):
        self.model_id = model_id
        self.do_dryrun = do_dryrun
        self.verbose = verbose
        self.render_inference = render_inference

        self.timeout = None

        self.url = f"https://api.ultralytics.com/v1/predict/{self.model_id}"
        logger.info(f"{self.__class__.__name__}.url: {self.url}")

        self.valid = True
        if not env.ULTRALYTICS_API_KEY:
            logger.error('ULTRALYTICS_API_KEY not found, update .env."')
            self.valid = False

        self.headers = {
            "x-api-key": env.ULTRALYTICS_API_KEY,
        }
        self.data = {
            "size": 640,
            "confidence": 0.25,
            "iou": 0.45,
        }

    def infer(
        self,
        image_filename: str,
    ) -> Tuple[bool, Dict]:
        if self.do_dryrun:
            return {}

        logger.info("{}.infer({})".format(self.__class__.__name__, image_filename))

        with open(image_filename, "rb") as f:
            response = requests.post(
                self.url,
                headers=self.headers,
                data=self.data,
                files={"image": f},
                timeout=self.timeout,
            )

        # https://chat.openai.com/c/6deb94d0-826a-48de-b5ef-f7d8da416c82
        # response.raise_for_status()
        if (
            response.status_code // 100 != 2
        ):  # Check if status code is not in the 2xx range
            logger.info(
                "{}.infer({}) failed, status_code: {}, reason: {}.".format(
                    NAME,
                    image_filename,
                    response.status_code,
                    response.reason,
                )
            )
            return False, {}

        response_dict = response.json()

        if self.verbose:
            print(json.dumps(response_dict, indent=2))

        summary = ", ".join(
            [
                "{}: {}".format(thing, count)
                for thing, count in Counter(
                    [thing["name"] for thing in response_dict["images"][0]["results"]]
                ).items()
            ]
        )
        logger.info(summary)

        if self.render_inference:
            render_filename = file.add_suffix(
                file.add_extension(image_filename, "jpg"), "inference"
            )

            file.save_image(
                render_filename,
                self.render(
                    image=file.load_image(image_filename)[1].copy(),
                    inference=response_dict,
                    header=[
                        " | ".join(
                            object_signature(
                                object_name=path.name(file.path(image_filename)),
                                info=file.name(image_filename),
                            )
                            + [summary]
                        )
                    ],
                ),
                log=True,
            )

            response_dict["render_filename"] = render_filename

        return True, response_dict

    def render(
        self,
        image,
        inference,
        header: List[str] = [],
        fontFace=cv2.FONT_HERSHEY_SIMPLEX,
        fontScale=0.5,
    ):
        image = image.copy()
        for thing in inference["images"][0]["results"]:
            try:
                x1 = int(thing["box"]["x1"])
                x2 = int(thing["box"]["x2"])
                y1 = int(thing["box"]["y1"])
                y2 = int(thing["box"]["y2"])
                text = "{}@{:.2f}".format(
                    thing["name"],
                    thing["confidence"],
                )

                image[y1:y2, x1:x2, :] = 255 - image[y1:y2, x1:x2, :]
                for thickness, color in zip([4, 1], [0, 255]):
                    image = cv2.putText(
                        image,
                        text=text,
                        org=(x2, y2),
                        fontFace=fontFace,
                        fontScale=fontScale,
                        color=3 * (color,),
                        thickness=thickness,
                    )
            except Exception as e:
                logger.warning(e)

        return add_signature(
            image,
            header=header,
            footer=[" | ".join(signature())],
            word_wrap=True,
        )



from typing import List

from blue_options.terminal import show_usage, xtra
from abcli.help.generic import help_functions as generic_help_functions

from vancouver_watching import ALIAS
from vancouver_watching.help.discover import help_discover
from vancouver_watching.help.ingest import help_ingest
from vancouver_watching.help.detect import help_detect


help_functions = generic_help_functions(plugin_name=ALIAS)

help_functions.update(
    {
        "detect": help_detect,
        "discover": help_discover,
        "ingest": help_ingest,
    }
)



import argparse

from blueness import module
from blueness.argparse.generic import sys_exit

from vancouver_watching import NAME
from vancouver_watching.target import Target
from vancouver_watching.target.detection import detect_in_target
from vancouver_watching.target.ingest import ingest_target
from vancouver_watching.logger import logger

NAME = module.name(__file__, NAME)

list_of_tasks = ["detect", "ingest"]

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help=" | ".join(list_of_tasks),
)
parser.add_argument(
    "--geojson",
    type=str,
    default="",
)
parser.add_argument(
    "--count",
    type=int,
    default=-1,
    help="-1: all",
)
parser.add_argument(
    "--do_dryrun",
    type=int,
    default=0,
    help="0|1",
)
parser.add_argument(
    "--verbose",
    type=int,
    default=0,
    help="0|1",
)
parser.add_argument(
    "--overwrite",
    type=int,
    default=0,
    help="0|1",
)
parser.add_argument(
    "--animated_gif",
    type=int,
    default=0,
    help="0|1",
)
parser.add_argument(
    "--model_id",
    type=str,
    default="",
)
args = parser.parse_args()

success = args.task in list_of_tasks

if success:
    target = Target(
        map_filename=args.geojson,
        do_dryrun=args.do_dryrun,
        verbose=args.verbose,
    )
    success = target.valid

if args.task == "detect":
    if success:
        success = detect_in_target(
            target=target,
            model_id=args.model_id,
            animated_gif=args.animated_gif,
            count=args.count,
            overwrite=args.overwrite,
        )
elif args.task == "ingest":
    if success:
        success = ingest_target(
            target=target,
            count=args.count,
        )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from tqdm import tqdm

from blue_objects import file, path
from blue_geo.file import load_geodataframe, save_geojson

from vancouver_watching.logger import logger


class Target:
    def __init__(
        self,
        map_filename: str,
        do_dryrun: bool = False,
        verbose: bool = False,
    ):
        self.map_filename = map_filename  # geojson file

        self.object_path = file.path(self.map_filename)
        self.object_name = path.name(self.object_path)
        success, self.gdf = load_geodataframe(self.map_filename)
        assert success

        self.do_dryrun = do_dryrun
        self.verbose = verbose

        self.metadata_filename = file.add_extension(self.map_filename, "json")
        self.metadata = {}

        self.valid = True
        for column in "cameras,mapid".split(","):
            if column not in self.gdf.columns:
                self.valid = False
                logger.info(
                    "Invalid target: {} not found in {}.".format(
                        column, self.map_filename
                    )
                )
                return

        logger.info(
            "{}: {} mapid(s) from {}.".format(
                self.__class__.__name__,
                len(self.gdf),
                self.map_filename,
            )
        )

        success, self.metadata = file.load_json(
            self.metadata_filename,
            ignore_error=True,
        )
        if not success:
            logger.info("generating metadata: {}".format(self.metadata_filename))

            for _, row in tqdm(self.gdf.iterrows()):
                cameras = {}
                for url in row["cameras"].split(","):
                    filename = file.name_and_extension(url)

                    if file.extension(filename) not in "jpg,jpeg,png".split(","):
                        logger.error("bad url: {}.".format(url))
                        continue

                    cameras[filename] = {"url": url}

                self.metadata[row["mapid"]] = {"cameras": cameras}

            self.save_metadata()

    def on_map(
        self,
        zoom_start: int = 12,
    ):
        import folium

        if self.gdf.empty:
            return None

        output = folium.Map(
            location=self.gdf.unary_union.centroid.coords[0][::-1],
            zoom_start=zoom_start,
        )
        folium.GeoJson(self.gdf).add_to(output)

        return output

    def save_gdf(self) -> bool:
        return save_geojson(
            self.map_filename,
            self.gdf,
            log=True,
        )

    def save_metadata(self) -> bool:
        return file.save_json(
            self.metadata_filename,
            self.metadata,
            log=True,
        )
